==> crow-2.11.0.tar.bz2: crow-2.11.0/designer/application.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_APPLICATION_H
#define CROW_APPLICATION_H

#ifdef CROW_SANDBOX
	#define G_DISABLE_DEPRECATED
	#define GDK_DISABLE_DEPRECATED
	#define GDK_PIXBUF_DISABLE_DEPRECATED
	#define PANGO_DISABLE_DEPRECATED
	#define GTK_DISABLE_DEPRECATED
	#define GTKMM_DISABLE_DEPRECATED
#endif

#include <gtk/gtk.h>

#ifdef _WIN32
	#include <gdk/gdkwin32.h>
	#include <shellapi.h>
#endif

#include <assert.h>

#include <crow.h>
#include <guiloader-c++.h>
#include <gtkmm/icontheme.h>

#include <vector>
#include <sstream>
#include <iostream>
#include <iomanip>

#ifdef G_LOG_DOMAIN
	#undef G_LOG_DOMAIN
#endif
#define G_LOG_DOMAIN "CrowApplication"

#define Check(expr) if(!(expr)) CheckFailed(#expr, __FILE__, __LINE__); else

namespace CrowDesignerApp {

typedef std::map<std::string, Glib::ustring> Settings;

void CheckFailed(const char * expr, const char * file, unsigned line);
void FatalError(const std::string & message);

std::vector<std::string> Arguments(int argc, char ** argv);
std::vector<Glib::ustring> Split(const Glib::ustring & str, gunichar ch);

GHashTable * SettingsToHashTable(const Settings & settings);
Settings HashTableToSettings(GHashTable * table);

int GetKeyInt(const Settings & settings, const std::string & key);
void SetKeyInt(Settings * settings, const std::string & key, int value);

Glib::ustring GetKeyString(const Settings & settings, const std::string & key);
void SetKeyString(Settings * settings, const std::string & key, const char * const value);
void EnsureKeyString(Settings * settings, const std::string & key, const char * const value);

bool GetKeyBool(const Settings & settings, const std::string & key);
void SetKeyBool(Settings * settings, const std::string & key, bool value);

Gdk::Color GetKeyColor(const Settings & settings, const std::string & key);
void SetKeyColor(Settings * settings, const std::string & key, const Gdk::Color & value);

class ErrorDialog : public Gtk::Window {

public:
	ErrorDialog();

	void onErrorMessage(const Glib::ustring & message);

private:
	Gtk::ScrolledWindow scroll;
	Gtk::TextView textView;

};

class SettingsDialog : public sigc::trackable {

public:
	SettingsDialog();

	void setSettings(const Settings & settings);
	Settings getSettings();

	Gtk::Dialog * getWindow() { return window; }

protected:
	void onAdd();
	void onRemove();
	void onMoveUp();
	void onMoveDown();
	void onColorChanged();
	void onCheckChanged();
	void onSelectionChanged();
	void onSwitchPage(GtkNotebookPage *, guint);
	void onDefaults();

	void updateUi(const Settings & settings);
	void updateButtons();
	void updateActions();

	Gtk::TreeIter getPosition();
	void actionCompleted();

private:
	Gui::PLoader loader;

	Settings original, defaults;

	Gtk::Dialog * window;	
	Gtk::Button * defaultsButton;
	Gtk::Notebook * widget;
	Gtk::ColorButton * validColor;
	Gtk::ColorButton * wrongColor;
	Gtk::CheckButton * showDebugWidgets;
	Gtk::CheckButton * showHiddenProperties;
	Gtk::CheckButton * suppressMessages;
	Glib::RefPtr<Gtk::ActionGroup> actions;
	Gtk::TreeView * folders;

	Glib::RefPtr<Gtk::ListStore> model;

};

class AppWindow : public Gtk::Window {

public:
	struct State {
		State() { width= height= maximized= -1; }
		int width;
		int height;
		int maximized;
	};

	State getState();
	void setState(const State & state);

protected:
	int getMaximized();
	Glib::RefPtr<Gdk::Window> getGdkWindow();

};

class Application : public sigc::trackable {

public:
	Application(const std::vector<std::string> & args1, const std::vector<std::string> & args2);
	~Application();

	Gtk::Window * getWindow() { return &window; }

protected:
	struct File {
		File() { header= false; }
		File(const std::string & fn, bool hdr=false) { name= fn; header= hdr; }
		bool empty() { return name.empty(); }
		void clear() { name.clear(); header= false; }
		std::string name;
		bool header;
	};

	void setupHandler(const char * domain);

	static void logHandler(
		const gchar * logDomain,
		GLogLevelFlags logLevel,
		const gchar * message,
		gpointer userData
	);

	void onErrorMessage(
		const Glib::ustring & domain,
		GLogLevelFlags level,
		const Glib::ustring & message
	);

	void createActionGroup();
	void createUIManager();
	void setupGtkAction(CrowActions actions, CrowActions changed, CrowActions ca, const char * str);

	static void on_actions_updated(CrowDesigner * obj, CrowActions changed, Application * app);
	static void on_status_updated(CrowDesigner * obj, const gchar * str, Application * app);
	static void on_info_updated(CrowDesigner * obj, const gchar * str, Application * app);
	static void on_show_url(CrowDesigner * obj, const gchar * str, Application * app);

	void onActionsUpdated(CrowActions changed);
	void onStatusUpdated(const Glib::ustring & str);
	void onInfoUpdated(const Glib::ustring & str);
	void onShowUrl(const Glib::ustring & str);
	
	void onClear();
	void onClone();
	void onOpen();
	void onOpenRecent(Gtk::RecentChooser * chooser);
	void openFile(const File & filename);
	void onSave();
	void onSaveAs();
	void onQuit();
	void onSettings();
	void onHome();
	void onApi();

	bool onDeleteEvent(GdkEventAny *);
	void onException();

	bool loadFile(const File & name, Glib::ustring * data);
	bool saveFile(const File & name, const Glib::ustring & data);
	File askFilename(Gtk::FileChooserAction action);
	void onCheckHeader(Gtk::FileChooser * chooser, Gtk::CheckButton * button);
	bool askProceed();
	void updateTitle();

	std::vector<Glib::ustring> currentIconPath();
	void updateIconPath();
	void loadKeys(Settings * skeys);
	void loadSettings();
	void saveSettings(bool quit=false);

	void addRecentFile();
	void showMessage(const Glib::ustring & message);
	void showUrl(const Glib::ustring & url);

	void designer_apply_settings(const Settings & settings);
	void designer_store_settings(Settings * settings);
	void designer_clear();
	void designer_load(const Glib::ustring & data);
	Glib::ustring designer_save();
	Glib::ustring designer_save_header();
	void designer_saved();
	Gtk::Widget * designer_get_widget();
	CrowActions designer_get_enabled_actions();
	void designer_run_action(CrowActions action);

private:
	std::vector<Glib::ustring> defaultIconPath;
	Settings settings;
	ErrorDialog errorDialog;

	typedef std::list<std::pair<const char *, guint> > Hids;
	Hids hids;
	
	std::string startupDirectory;
	std::vector<std::string> startupArgs1, startupArgs2;

	File file;

	AppWindow window;
	CrowDesigner * designer;

	Glib::RefPtr<Gtk::UIManager> uiManager;
	Glib::RefPtr<Gtk::ActionGroup> actionGroup;

	Gtk::VBox vBox;
	Gtk::Statusbar statusbar;
	Gtk::Frame infoFrame;
	Gtk::Label infoLabel;

};

}

#endif

==> crow-2.11.3.tar.bz2: crow-2.11.3/designer/application.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2008 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_APPLICATION_H
#define CROW_APPLICATION_H

#ifdef CROW_SANDBOX
	#define G_DISABLE_DEPRECATED
	#define GDK_DISABLE_DEPRECATED
	#define GDK_PIXBUF_DISABLE_DEPRECATED
	#define PANGO_DISABLE_DEPRECATED
	#define GTK_DISABLE_DEPRECATED
	#define GTKMM_DISABLE_DEPRECATED
#endif

#include <gtk/gtk.h>

#ifdef _WIN32
	#include <gdk/gdkwin32.h>
	#include <shellapi.h>
#endif

#include <assert.h>
#include <sys/stat.h>

#include <crow.h>
#include <guiloader-c++.h>

#include <vector>
#include <sstream>
#include <iostream>
#include <iomanip>

#ifdef G_LOG_DOMAIN
	#undef G_LOG_DOMAIN
#endif
#define G_LOG_DOMAIN "CrowApplication"

#define Check(expr) ((expr) ? NoObj() : (CheckFailed(#expr, __FILE__, __LINE__), NoObj()))

namespace CrowDesignerApp {

struct NoObj { };

typedef std::map<std::string, Glib::ustring> Settings;

typedef Glib::RefPtr<Gtk::RecentAction> PGtkRecentAction;

void CheckFailed(const char * expr, const char * file, unsigned line);
void FatalError(const std::string & message);

std::vector<std::string> Arguments(int argc, char ** argv);
std::vector<Glib::ustring> Split(const Glib::ustring & str, gunichar ch);

GHashTable * SettingsToHashTable(const Settings & settings);
Settings HashTableToSettings(GHashTable * table);

int GetKeyInt(const Settings & settings, const std::string & key);
void SetKeyInt(Settings * settings, const std::string & key, int value);

Glib::ustring GetKeyString(const Settings & settings, const std::string & key);
void SetKeyString(Settings * settings, const std::string & key, const char * const value);
void EnsureKeyString(Settings * settings, const std::string & key, const char * const value);

bool GetKeyBool(const Settings & settings, const std::string & key);
void SetKeyBool(Settings * settings, const std::string & key, bool value);

Gdk::Color GetKeyColor(const Settings & settings, const std::string & key);
void SetKeyColor(Settings * settings, const std::string & key, const Gdk::Color & value);

class ErrorDialog : public Gtk::Window {

public:
	ErrorDialog();

	void onErrorMessage(const Glib::ustring & message);

private:
	Gtk::ScrolledWindow scroll;
	Gtk::TextView textView;

};

class SettingsDialog : public sigc::trackable {

public:
	SettingsDialog();

	void setSettings(const Settings & settings);
	Settings getSettings();

	Gtk::Dialog * getWindow() { return window; }

protected:
	void onAdd();
	void onRemove();
	void onMoveUp();
	void onMoveDown();
	void onColorChanged();
	void onCheckChanged();
	void onSelectionChanged();
	void onSwitchPage(GtkNotebookPage *, guint);
	void onDefaults();

	void updateUi(const Settings & settings);
	void updateButtons();
	void updateActions();

	Gtk::TreeIter getPosition();
	void actionCompleted();

private:
	Gui::PLoader loader;

	Settings original, defaults;

	Gtk::Dialog * window;	
	Gtk::Button * defaultsButton;
	Gtk::Notebook * widget;
	Gtk::ColorButton * validColor;
	Gtk::ColorButton * wrongColor;
	Gtk::CheckButton * showDebugWidgets;
	Gtk::CheckButton * showHiddenProperties;
	Gtk::CheckButton * suppressMessages;
	Glib::RefPtr<Gtk::ActionGroup> actions;
	Gtk::TreeView * folders;

	Glib::RefPtr<Gtk::ListStore> model;

};

class AppWindow : public Gtk::Window {

public:
	struct State {
		State() { width= height= maximized= -1; }
		int width;
		int height;
		int maximized;
	};

	State getState();
	void setState(const State & state);

protected:
	int getMaximized();
	Glib::RefPtr<Gdk::Window> getGdkWindow();

};

class Application : public sigc::trackable {

public:
	Application(const std::vector<std::string> & args1, const std::vector<std::string> & args2);
	~Application();

	Gtk::Window * getWindow() { return &window; }

protected:
	struct File {
		File() { header= false; }
		File(const std::string & fn, bool hdr=false) { name= fn; header= hdr; }
		bool empty() { return name.empty(); }
		void clear() { name.clear(); header= false; }
		std::string name;
		bool header;
	};

	void setupHandler(const char * domain);

	static void logHandler(
		const gchar * logDomain,
		GLogLevelFlags logLevel,
		const gchar * message,
		gpointer userData
	);

	void onErrorMessage(
		const Glib::ustring & domain,
		GLogLevelFlags level,
		const Glib::ustring & message
	);

	void createActionGroup();
	void createUIManager();
	void setupGtkAction(CrowActions actions, CrowActions changed, CrowActions ca, const char * str);

	static void on_actions_updated(CrowDesigner * obj, CrowActions changed, Application * app);
	static void on_status_updated(CrowDesigner * obj, const gchar * str, Application * app);
	static void on_info_updated(CrowDesigner * obj, const gchar * str, Application * app);
	static void on_show_url(CrowDesigner * obj, const gchar * str, Application * app);

	void onActionsUpdated(CrowActions changed);
	void onStatusUpdated(const Glib::ustring & str);
	void onInfoUpdated(const Glib::ustring & str);
	void onShowUrl(const Glib::ustring & str);
	
	void onClear();
	void onClone();
	void onOpen();
	void onOpenRecent();
	void openFile(const File & filename);
	void onSave();
	void onSaveAs();
	void onQuit();
	void onSettings();
	void onHome();
	void onApi();

	bool onDeleteEvent(GdkEventAny *);
	void onException();

	bool loadFile(const File & name, Glib::ustring * data);
	bool saveFile(const File & name, const Glib::ustring & data);
	File askFilename(Gtk::FileChooserAction action);
	void onCheckHeader(Gtk::FileChooser * chooser, Gtk::CheckButton * button);
	bool askProceed();
	void updateTitle();

	std::vector<Glib::ustring> currentIconPath();
	void updateIconPath();
	std::string getConfigPath();
	void loadKeys(Glib::KeyFile * file, Settings * skeys);
	void loadSettings();
	void saveSettings(bool quit=false);

	void addRecentFile();
	void showMessage(const Glib::ustring & message);
	void showUrl(const Glib::ustring & url);

	void designer_apply_settings(const Settings & settings);
	void designer_store_settings(Settings * settings);
	void designer_clear();
	void designer_load(const Glib::ustring & data);
	Glib::ustring designer_save();
	Glib::ustring designer_save_header();
	void designer_saved();
	Gtk::Widget * designer_get_widget();
	CrowActions designer_get_enabled_actions();
	void designer_run_action(CrowActions action);

private:
	std::vector<Glib::ustring> defaultIconPath;
	Settings settings;
	ErrorDialog errorDialog;

	typedef std::list<std::pair<const char *, guint> > Hids;
	Hids hids;
	
	std::string startupDirectory;
	std::vector<std::string> startupArgs1, startupArgs2;

	File file;

	AppWindow window;
	CrowDesigner * designer;

	Glib::RefPtr<Gtk::UIManager> uiManager;
	Glib::RefPtr<Gtk::ActionGroup> actionGroup;

	Gtk::VBox vBox;
	Gtk::Statusbar statusbar;
	Gtk::Frame infoFrame;
	Gtk::Label infoLabel;

};

}

#endif
